<?php

namespace App\Http\Controllers;

use App\Http\Requests\GradeSubjectPostRequest;
use App\Models\Grade;
use App\Models\Subject;
use App\Services\FileService;
use App\Services\JournalService;
use Illuminate\Http\RedirectResponse;
use Illuminate\View\View;

class GradeSubjectController extends Controller
{
    public function create(Grade $grade): View
    {
        return view('grade-subject.create', [
            'grade' => $grade,
            'subjects' => Subject::all(),
        ]);
    }

    public function store(GradeSubjectPostRequest $request, Grade $grade): RedirectResponse
    {
        $grade->subjects()->syncWithoutDetaching($request->subject_id);

        return redirect()->route('grades.show', $grade);
    }

    public function destroy(Grade $grade, Subject $subject): RedirectResponse
    {
        $grade->subjects()->detach($subject);

        return redirect()->route('grades.show', $grade);
    }

    public function journal(Grade $grade, Subject $subject, JournalService $service): View
    {
        return view('grade-subject.journal', [
            'lessons' => $grade->lessons()->where('subject_id', $subject->id)->with('students')->get(),
            'students' => $grade->students()->orderBy('last_name')->get(),
            'grade' => $grade,
            'subject' => $subject,
        ]);
    }

    public function exportToExcel(Grade $grade, Subject $subject, FileService $service)
    {
        return $service->exportJournal($grade, $subject);
    }
}
